
viewer.mode=FULLSCREEN_NO_BUTTONS

function setup() 
    assert(craft, "Please include Craft as a dependency") 
    if terrainModel then terrainModel = nil end
    hasDrawnOnce = false
end

function runStartupFunctions()
    createScene() 
    createLightConditions()
    createStartConditions()
    createExplosions()
    createStartingSpheres()   
    createShip()
    setFinalSettings()
end

function createStartConditions()
    maxSpeed=1.6
    count=100
    ships=4
    youWon,hitRed,youLost=false,false,false
    redBalls,greenBalls,yellowBalls=0,0,0
    hgx=Gravity.x
    speed,ey,ang=0,45,0
    cameraX,cameraZ=-205,-205
    currentPresets = "dave1707"
    use3DShip = false
    use3DExplosions = false
    explodeys = {}
    sphereMaterial = craft.material(asset.builtin.Materials.Specular)
    shardMaterial = craft.material(asset.builtin.Materials.Basic)
end

function createScene()
    scene = craft.scene()
    scene.camera.position = vec3(cameraX,0,cameraZ)
    scene.camera.eulerAngles=vec3(ex,ey,ez)
end

function createLightConditions()
    scene.sun.rotation = quat.eulerAngles(45,0,45)   
    scene.ambientColor = color(90,90,90)   
    --dont matter because sky is inactive:
    skyMaterial = scene.sky.material
    skyMaterial.sky = color(20, 16, 15)
    skyMaterial.horizon = color(10, 11, 12)    
    cameraComponent = scene.camera:get(craft.camera)
    cameraComponent.hdr = true
    cameraComponent.colorTextureEnabled = true
    sunLight = scene.sun:get(craft.light)
    bloom = craft.bloomEffect()
    cameraComponent:addPostEffect(bloom)
    bloom.threshold = 4
    bloom.intensity = 1.2
    bloom.softThreshold = 0.4
    bloom.iterations = 8
    scene.sky.active = true
end

function createMaterialsForGlows()
    redNormalMaterial = craft.material(asset.builtin.Materials.Specular) 
    redNormalMaterial.diffuse=color(255, 0, 0)    
    greenNormalMaterial = craft.material(asset.builtin.Materials.Specular) 
    greenNormalMaterial.diffuse = color(0, 255, 0)
    greenGlowyMaterial = craft.material(asset.builtin.Materials.Basic) 
    greenGlowyMaterial.diffuse = vec3(80/255, 200/255, 120/255) * 0.5
    yellowNormalMaterial = craft.material(asset.builtin.Materials.Specular) 
    yellowNormalMaterial.diffuse = color(255, 255, 0)
    yellowGlowyMaterial = craft.material(asset.builtin.Materials.Basic) 
    yellowGlowyMaterial.diffuse = vec3(255/255, 225/255, 120/255) * 9.75  
end

function createStartingSpheres()
    createMaterialsForGlows()
    tab={}
    for z=1,count do    -- 1=red
        createSphere(math.random(-200,200),math.random(-200,200),2,1,255,0,0,0,0)
        redBalls=redBalls+1
    end
    for z=1,count do    -- 2=green  
        createSphere(math.random(-200,200),math.random(-200,200),.5,2,0,255,0,0,0)
        greenBalls=greenBalls+1
    end
end

function createExplosions()
    explosion = Explosion(scene)
    explosion2 = Explosion(scene)
end

function colorToDiffusion(colorGiven)
    local colorAsVec3 = vec3(colorGiven.r, colorGiven.g, colorGiven.b)
    return colorAsVec3 / 15
end

function setMaterialForSphereType(sphereType, material)
    for k, v in pairs(tab) do
        if v.type == sphereType then
            v.ent.material = material
        end
    end
end

function setFinalSettings()
        greenBallDiffusionToggle = true
        greenBallDiffusion = vec3(55/255, 250/255, 90/255) * 3.75
        setMaterialForSphereType(2, greenGlowyMaterial)
        greenGlowyMaterial.diffuse = greenBallDiffusion
        yellowBallDiffusionToggle = true
        yellowBallDiffusion = vec3(255/255, 225/255, 120/255) * 9.75
        setMaterialForSphereType(3, yellowGlowyMaterial)
        yellowGlowyMaterial.diffuse = yellowBallDiffusion
        sunIntensity = 3
        sunPosX = -8
        sunPosY = -26
        sunPosZ = -66
        use3DShip = true
        ship.scale = vec3(1,1,1) * 0.018
        use3DExplosions = true
end

function createShip()
    ship = scene:entity()
    ship.model = craft.model(asset.builtin.SpaceKit.spaceCraft6_obj)
    print(ship.scale)
    if use3DShip then
        ship.scale = ship.scale * 0.018
    else
        ship.scale = ship.scale * 0.0
    end
    ship.parent = scene.camera
    ship.position = ship.position + vec3(0,-HEIGHT*0.0003,HEIGHT*0.0015)    
end

function createFloor(x,z)
    c1=scene:entity()
    c1.model = craft.model.cube(vec3(400,1,400))
    c1.position=vec3(x,-1.25,z)
    c1.material = craft.material(asset.builtin.Materials.Standard)
    c1.material.map = readImage(asset.builtin.Surfaces.Desert_Cliff_Color)
    c1.material.offsetRepeat=vec4(0,0,30,30)
    c1.material.roughness = 100
    c1.material.roughnessMap = readImage(asset.builtin.Surfaces.Desert_Cliff_Roughness)
    c1.material.normalMap = readImage(asset.builtin.Surfaces.Desert_Cliff_Normal)
    c1.material.aoMap = readImage(asset.builtin.Surfaces.Desert_Cliff_AO)
    c1.material.aoMapIntensity = 2.5
end

function createCraftTerrain()
    if terrainModel then return end
    local x = x or 0
    local z = z or 0
    local seed = math.random(5000)
    terrainModel = makeCraftTerrain(600,400,13.25,seed,math.random(2,6))
    ct=scene:entity()
    ct.model = terrainModel
    ct.position = vec3(x - 400 / 2, -17, z - 400 / 2) -- Adjust as necessary
    ct.material = craft.material(asset.builtin.Materials.Basic)
    ct.material.map = readImage(asset.builtin.Surfaces.Desert_Cliff_Color)
    ct.material.roughness = 100
    ct.material.roughnessMap = readImage(asset.builtin.Surfaces.Desert_Cliff_Roughness)
    ct.material.normalMap = readImage(asset.builtin.Surfaces.Desert_Cliff_Normal)
    ct.material.aoMap = readImage(asset.builtin.Surfaces.Desert_Cliff_AO)
    ct.material.aoMapIntensity = 0.5
    ct.material.offsetRepeat=vec4(0,0,13,32)
end

function update(dt)
    scene:update(dt)
    if not hitRed then
        scene.camera.position = vec3(cameraX,1,cameraZ)
        scene.camera.eulerAngles=vec3(0,ey,0)
    end
    --I would love to destroy exploded objects instead of 
    --just making them inactive, but I can't find a way to do 
    --it without crashing Codea
    for i = #explodeys, 1, -1 do
        local sphere = explodeys[i]
        if sphere then
            sphere.explodeOneFrame(1)
            if sphere.timeSinceExplosionStarted >= sphere.explodeDuration then
                -- Iterate backwards over the shards to safely destroy and remove them
                for j = #sphere.shards, 1, -1 do
                    local shard = sphere.shards[j]
                    if shard.active then
                        sphere.shards[j] = nil
                        shard.active = false
                    end
                end
                explodeys[i] = nil
                sphere.entityHolder.active = false
                sphere.active = false
            end   
        end     
    end

    scene.sun.rotation = quat.eulerAngles(sunPosX, sunPosY, sunPosZ)
    --scene.sky.material.sky = skyColor
    --scene.sky.material.horizon = horizonColor
    sunLight.intensity = sunIntensity
end

function draw()
    if not terrainModel then
      --  background(19, 19, 27)
        local startImg = readImage(asset.StartScreen)
        sprite(startImg, WIDTH/2, HEIGHT/2, WIDTH, WIDTH * startImg.height / startImg.width)
        pushStyle()
        fontSize(36)
        fill(255, 148, 0)
        text("Generating terrain",WIDTH/2,HEIGHT*0.7)
        fontSize(24)
        fill(227, 169, 110)
        text("Takes around 10 seconds",WIDTH/2,HEIGHT*.7-90)

        --runStartupFunctions()
        popStyle()
        if not hasDrawnOnce then
            hasDrawnOnce = true
            return
        else
            runStartupFunctions()
            createCraftTerrain()
        end
    end
    update(DeltaTime)   
    scene:draw()  
    if paused then  
        pushStyle()
        fontSize(30)
        text("PAUSED",WIDTH/2,HEIGHT*.65-40)
        fill(227, 169, 110)
        fontSize(24)
        text("Triple tap to restart",WIDTH/2,HEIGHT*.55)
        popStyle()
    else
        updateCameraPos()        
        checkCollisions()
        checkTilt() 
        drawShip()
        drawShipLevels()
    end
    if youLost then
        youLostFunc()
    elseif hitRed then
        hitRedFunc()
    elseif youWon then
        youWonFunc()
    end
end

function touched(t)
    if t.state==BEGAN then
        if t.tapCount==3 then
            ang=0
            paused=not paused
        end
        if t.tapCount==2 then
            if youWon then
                setup()
            elseif youLost then
                setup()
            elseif hitRed then
                hitRed=false
                hgx=Gravity.x
                cameraX,cameraZ=-205,-205
                ships=ships-1
                speed,ey,ang=0,45,0
                if use3DShip then
                    ship.scale = vec3(1,1,1) * 0.018
                end
                if explosion.pt then
                    explosion.pt:destroy()
                end
                if explosion2.pt then
                    explosion2.pt:destroy()
                end
                if explosionSounds then
                    for _, source in pairs(explosionSounds) do
                        source.muted = true
                    end
                    explosionSounds = nil
                end
            end
        else
            ang=0
            if speed==0 then
                speed=maxSpeed
            end
        end
    end
end

function youLostFunc()
    background(151, 41, 27)
    pushStyle()
    fontSize(48)
    fill(255,0,0)
    text("YOU LOST!",WIDTH/2,HEIGHT*.65)
    fontSize(30)
    fill(227, 169, 110)
    text("Hold device level",WIDTH/2,HEIGHT*.65-90)
    text("Double tap to play again",WIDTH/2,HEIGHT*.65-140)
    popStyle()
end

function hitRedFunc()
    hgx=Gravity.x
    speed = 0
    if use3DExplosions then
        explosion:draw()
        explosion2:draw()
    else
        background(0)
        sprite(asset.builtin.Tyrian_Remastered.Explosion_Huge,WIDTH/2,HEIGHT/2,500,500)
    end
    fill(255,0,0)
    if use3DExplosions then
        fill(230, 236, 67)
    end
    pushStyle()
    fontSize(30)
    fill(230, 236, 67)
    text("Hold device level",WIDTH/2,HEIGHT*.65-40)
    fontSize(24)
    fill(227, 169, 110)
    text("Double tap to continue",WIDTH/2,HEIGHT*.65-90)
    popStyle()
end

function youWonFunc()
    background(255, 240, 0)
    pushStyle()
    fontSize(48)
    fill(0, 124, 255)
    text("YOU WON!",WIDTH/2,HEIGHT*.65)
    fontSize(30)
    fill(43, 193, 193)
    text("Hold device level",WIDTH/2,HEIGHT*.65-90)
    text("Double tap to restart",WIDTH/2,HEIGHT*.65-140)
    popStyle()
end

function drawShipLevels()
    for z=1,ships do
        sprite(asset.builtin.Tyrian_Remastered.Boss_A,z*50,60,40)
    end
end

function updateCameraPos()
    ey=ey-ang
    x=speed*math.sin(math.rad(ey))
    z=speed*math.cos(math.rad(ey)) 
    cameraX=cameraX+x
    cameraZ=cameraZ+z
end

-- Function to start the flashing effect
function flashSkyColors(sky, horizon)
    local originalColorH = skyMaterial.horizon
    local originalColorS = skyMaterial.sky
    local flashColorH = horizon or color(223, 234, 222)
    local flashColorS = sky or color(236, 158, 67)
    local flashSpeed = 0.01
    tween(flashSpeed, skyMaterial, {}, tween.easing.linear, function()
        scene.sky.material.sky = flashColorS
        scene.sky.material.horizon = flashColorH
        tween(flashSpeed, skyMaterial, {}, tween.easing.linear, function()
            scene.sky.material.sky = originalColorS
            scene.sky.material.horizon = originalColorH
            tween(flashSpeed, skyMaterial, {}, tween.easing.linear, function()
                scene.sky.material.sky = flashColorS
                scene.sky.material.horizon = flashColorH
                tween(flashSpeed, skyMaterial, {}, tween.easing.linear, function()
                    scene.sky.material.sky = originalColorS
                    scene.sky.material.horizon = originalColorH
                    tween(flashSpeed, skyMaterial, {}, tween.easing.linear, function()
                        scene.sky.material.sky = flashColorS
                        scene.sky.material.horizon = flashColorH
                        tween(flashSpeed, skyMaterial, {}, tween.easing.linear, function()
                            scene.sky.material.sky = originalColorS
                            scene.sky.material.horizon = originalColorH
                            tween(flashSpeed, skyMaterial, {}, tween.easing.linear, function()
                                scene.sky.material.sky = flashColorS
                                scene.sky.material.horizon = flashColorH
                                tween(flashSpeed, skyMaterial, {}, tween.easing.linear, function()
                                    scene.sky.material.sky = originalColorS
                                    scene.sky.material.horizon = originalColorH
                                end)
                            end)
                        end)
                    end)
                end)
            end)
        end)
    end)
end

function checkCollisions()
    for a,b in pairs(tab) do
        if b.type==1 then
            if cameraX>=b.ent.position.x-b.size and 
            cameraX<=b.ent.position.x+b.size and 
            cameraZ>=b.ent.position.z-b.size and
            cameraZ<=b.ent.position.z+b.size then
                if ships==0 then
                    youLost=true
                else
                    hitRed = true
                    ship.scale = vec3(0,0,0)
                    table.insert(explodeys, b.ent)
                    if use3DExplosions then
                        explosion:explodeAt(b.ent.position, vec3(1,1,1) * 0.5)
                        explosion2:explodeAt(b.ent.position, vec3(1,1,1) * 0.5)
                    end
                    scene.camera.position = b.ent.position + vec3(0, 10, -50)
                    scene.camera.eulerAngles=vec3(12, 0, math.random(-2, 3))
                end             
                --sound seed: 27037  
                if not explosionSounds then 
                    explosionSounds = {
                        sound(asset.downloaded.A_Hero_s_Quest.FireBall_Blast_1),
                        sound(asset.downloaded.A_Hero_s_Quest.FireBall_Blast_2),
                        sound(asset.downloaded.A_Hero_s_Quest.FireBall_Woosh)
                    }
                end
            end
        end
        if b.type==2 then
            if cameraX>b.ent.position.x-b.size and 
            cameraX<b.ent.position.x+b.size and
            cameraZ>b.ent.position.z-b.size and 
            cameraZ<b.ent.position.z+b.size then
                flashSkyColors()
                table.insert(explodeys, b.ent)
                table.remove(tab,a)
                count=count-1
                sound(asset.downloaded.Game_Sounds_One.Assembly_1)
                sound(asset.downloaded.Game_Sounds_One.Blaster)
                sound(asset.downloaded.Game_Sounds_One.Block_3)
                sound(asset.downloaded.A_Hero_s_Quest.Sword_Hit_3)
                createSphere(math.random(-200,200),
                math.random(-200,200),.5,3,255,255,0,.2,.2)
                yellowBalls=yellowBalls+1
                greenBalls=greenBalls-1
                if yellowBallDiffusionToggle then
                    setMaterialForSphereType(3, yellowGlowyMaterial)
                else
                    setMaterialForSphereType(3, yellowNormalMaterial)
                end
            end
        end
        if b.type==3 then
            if cameraX>=b.ent.position.x-b.size and 
            cameraX<=b.ent.position.x+b.size and 
            cameraZ>=b.ent.position.z-b.size and
            cameraZ<=b.ent.position.z+b.size then
                flashSkyColors(color(255, 129, 0), color(255, 236, 0))
                table.insert(explodeys, b.ent)
                table.remove(tab,a)
                yellowBalls=yellowBalls-1
                sound(asset.downloaded.Game_Sounds_One.Assembly_1)
                sound(asset.downloaded.Game_Sounds_One.Blaster)
                sound(asset.downloaded.Game_Sounds_One.Block_3)
                sound(asset.downloaded.A_Hero_s_Quest.Sword_Hit_3)
                yellowHit=true
            else
                xx=b.ent.position.x
                zz=b.ent.position.z
                xx=xx+b.xv
                zz=zz+b.zv
                if xx<-200 or xx>200 then
                    b.xv=-b.xv
                end
                if zz<-200 or zz>200 then
                    b.zv=-b.zv
                end
                b.ent.position=vec3(xx,1,zz)
            end
        end
    end
    if yellowHit then   -- remove a red ball
        yellowHit=false
        for z=#tab,1,-1 do
            if tab[z].type==1 then
                tab[z].ent:destroy()
                table.remove(tab,z)
                redBalls=redBalls-1
                break
            end
        end
    end
    if redBalls+greenBalls+yellowBalls==0 then
        youWon=true
    end
    if speed==0 and not hitRed then
        pushStyle()
        fontSize(30)
        fill(230, 236, 67)
        text("Tap screen to start",WIDTH/2,HEIGHT*.65-40)
            fontSize(24)
        fill(227, 169, 110)
            text("Triple tap to pause",WIDTH/2,HEIGHT*.65-90)
        popStyle()
    end
end

function checkTilt()
    gx=Gravity.x
    ang=ang+(gx-hgx)*4
    hgx=gx
    if gx>-.001 and gx<.001 then
        ang=0
    end
end

function drawShip()
    if not hitRed then
        pushMatrix()
        translate(WIDTH/2,HEIGHT*0.255)
        rotate(ang*-30)
        
        if not use3DShip then
            sprite(asset.builtin.Tyrian_Remastered.Boss_A,0,0,300)
        end
        fill(255,0,0)
        text(redBalls,-40,0)
        fill(0,255,0)
        text(greenBalls,0,0)
        fill(255,255,0)
        text(yellowBalls,40,0)   
        
        translate()
        popMatrix()
    end 
    ship.eulerAngles = vec3(0,-135, -ang*20)
    ship.eulerAngles = vec3(0,180, -ang*30)
end

function drawShip()
    if not hitRed then
        pushMatrix()
        translate(WIDTH/2,HEIGHT*0.255)
        rotate(ang*-30)
        
        -- Prepare to draw the thick line with rounded ends
        stroke(0, 0, 0, 128) -- Semi-transparent black
        strokeWidth(40) -- Set the stroke width to make it thick
        lineCapMode(ROUND) -- Ensure the line ends are rounded
        
        -- Draw the line
        line(-50, 0, 50, 0) -- Adjust the start and end points as needed
        
        noStroke() -- Stop applying stroke to subsequent drawings
        
        if not use3DShip then
            sprite(asset.builtin.Tyrian_Remastered.Boss_A,0,0,300)
        end
        
        -- Ensure text color is visible against the line's background
        fill(255, 0, 0) -- Red
        text(redBalls, -40, 0)
        fill(0, 255, 0) -- Green
        text(greenBalls, 0, 0)
        fill(255, 255, 0) -- Yellow
        text(yellowBalls, 40, 0)   
        
        popMatrix()
    end 
    ship.eulerAngles = vec3(0,-135, -ang*20)
    ship.eulerAngles = vec3(0,180, -ang*30)
end

function createSphere(x,z,size,type,r,g,b,xv,zv)
    sphere1=scene:entity()
    sphere1.model = craft.model.icosphere(size,1)
    sphere1.position=vec3(x,1,z)
    sphere1.material = redNormalMaterial
    if type == 2 then       
        explodeyCraft(sphere1, greenNormalMaterial, 2, 0.00006)
        sphere1.material = greenNormalMaterial
    elseif type == 3 then
        explodeyCraft(sphere1, yellowNormalMaterial, 2, 0.00006)
        sphere1.material = yellowNormalMaterial
    else --type 1
        explodeyCraft(sphere1, redNormalMaterial, 120) 
    end
    table.insert(tab,{ent=sphere1,xv=xv,zv=zv,size=size,type=type})
end 